import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:aorun_ad/aorun_ad.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatefulWidget {
  const MyApp({super.key});

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _platformVersion = 'Unknown';
  final _aorunAdPlugin = AorunAd();

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion;
    // Platform messages may fail, so we use a try/catch PlatformException.
    // We also handle the message potentially returning null.
    try {
      platformVersion =
          await _aorunAdPlugin.getPlatformVersion() ?? 'Unknown platform version';
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: Center(
          child: Column(
            children: [ElevatedButton(
              child: Text('激励视频广告（进阶）'),
              onPressed: () {
                showRewardVideoAd2();
                print("激励视频广告");
              },
            ),ElevatedButton(
              child: Text('插屏广告（进阶）'),
              onPressed: () {
                showChaPingAd();
                print("插屏广告");
              },
            )],
          ),
        ),
      ),
    );
  }

  /// 展示激励视频广告（进阶）
  Future<void> showRewardVideoAd2() async {
    try {
      String result = (await _aorunAdPlugin.showRewardVideoAd2()) as String;
      print("result"+result);
      print("3333");
    } on PlatformException catch (e) {
      print("调用广告错误");
    }
  }

  /// 展示插屏广告（进阶）
  Future<void> showChaPingAd() async {
    try {
      _aorunAdPlugin.showChaPingAd();
    } on PlatformException catch (e) {
      print("调用广告错误");
    }
  }
}
