//
//  XYUUID.m
//  XYUUID
//
//  Created by steve on 2016/7/21.
//  Copyright © 2020 guojunliu.github.io. All rights reserved.
//

#import "ZhiYuan_XYUUID.h"
#import <AdSupport/AdSupport.h>
#import <UIKit/UIKit.h>
#import "ZhiYuan_XYDeviceInfoUUID.h"
#import "ZhiYuan_XYKeyChain.h"
#if __has_include(<AppTrackingTransparency/AppTrackingTransparency.h>)
    #import <AppTrackingTransparency/AppTrackingTransparency.h>
#endif


static NSString *zhiyuan_ZeroIdfa = @"00000000-0000-0000-0000-000000000000";

static NSString *zhiyuan_XYUUIDForInstall = @"XYUUID-Install";
static NSString *zhiyuan_XYUUIDForKeyChain = @"XYUUID-KeyChain-ServiceDomain";
static NSString *zhiyuan_XYUUIDForKeyChainAndDeviceOrIdfa = @"XYUUID-KeyChain-Device-Idfa";
//static NSString *STR_XYUUIDForKeyChain = base64Decode:@"Y29tLkhvbGF2ZXJzZS5Ib2xhU3RhdGlzdGljYWw=";

@interface ZhiYuan_XYUUID ()
@property (nonatomic,copy) NSString *appOpenUUID;
@end

@implementation ZhiYuan_XYUUID

+ (instancetype)zhiyuan_shareManager{
    static ZhiYuan_XYUUID *instantce = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        if (instantce == nil) {
            instantce = [[ZhiYuan_XYUUID alloc] init];
        }
    });
    return instantce;
}

+ (NSString *)zhiyuan_uuid {
    return [[NSUUID UUID] UUIDString];
}

+ (NSString *)zhiyuan_uuidForInstall {
    NSString *uuid = [[NSUserDefaults standardUserDefaults] objectForKey:zhiyuan_XYUUIDForInstall];
    if (uuid && ![uuid isEqualToString:@""]) {
        return uuid;
    }
    uuid = [self zhiyuan_uuid];
    [[NSUserDefaults standardUserDefaults] setObject:uuid forKey:zhiyuan_XYUUIDForInstall];
    return uuid;
}

+ (NSString *)zhiyuan_uuidForAppOpen {
    if ([ZhiYuan_XYUUID zhiyuan_shareManager].appOpenUUID && ![[ZhiYuan_XYUUID zhiyuan_shareManager].appOpenUUID isEqualToString:@""]) {
        return [ZhiYuan_XYUUID zhiyuan_shareManager].appOpenUUID;
    }
    NSString *appOpenUUID = [self zhiyuan_uuid];
    [ZhiYuan_XYUUID zhiyuan_shareManager].appOpenUUID = appOpenUUID;
    return appOpenUUID;
}

+ (NSString *)zhiyuan_uuidForIDFA {
    
    if (@available(iOS 14.0, *)) {
        ATTrackingManagerAuthorizationStatus states = [ATTrackingManager trackingAuthorizationStatus];
        if (states == ATTrackingManagerAuthorizationStatusNotDetermined) {
            // 未提示用户
            return zhiyuan_ZeroIdfa;
        }
        else if (states == ATTrackingManagerAuthorizationStatusRestricted) {
            // 限制使用
            return zhiyuan_ZeroIdfa;
        }
        else if (states == ATTrackingManagerAuthorizationStatusDenied) {
            // 拒绝
            return zhiyuan_ZeroIdfa;
        }
        else if (states == ATTrackingManagerAuthorizationStatusAuthorized) {
            NSString *idfa = [[ASIdentifierManager sharedManager].advertisingIdentifier UUIDString];
            return idfa;
        }
        return zhiyuan_ZeroIdfa;
    }
    
    if (@available(iOS 10.0, *)) {
        BOOL canUseIDFA = [ASIdentifierManager sharedManager].advertisingTrackingEnabled;   //如果返回的YES说明没有 “开启限制广告跟踪”，可以获取到正确的idfa  如果返回的是NO，说明等待你的就是00000000-0000-0000-0000-000000000000
        if (canUseIDFA) {
            NSString *idfa = [[[ASIdentifierManager sharedManager] advertisingIdentifier] UUIDString];
            return idfa;
        }
        return zhiyuan_ZeroIdfa;
    }
    
    // iOS 10以下关闭广告追踪，也可以正常获取到IDFA
    NSString *idfa = [[[ASIdentifierManager sharedManager] advertisingIdentifier] UUIDString];
    return idfa;
}

+ (NSString *)zhiyuan_uuidForIDFV {
    NSString *idfv = [UIDevice currentDevice].identifierForVendor.UUIDString;
    return idfv;
}

+ (NSString *)zhiyuan_uuidForDeviceInfo {
    NSString *uuid = [ZhiYuan_XYDeviceInfoUUID zhiyuan_createDeviceInfoUUID];
    return uuid;
}

+ (NSString *)zhiyuan_uuidForKeychain {
    NSString *uuid = [ZhiYuan_XYKeyChain zhiyuan_getDataWithServiceDomain:zhiyuan_XYUUIDForKeyChain];
    if (uuid && ![uuid isEqualToString:@""]) {
        return uuid;
    }
    uuid = [self zhiyuan_uuid];
    [ZhiYuan_XYKeyChain zhiyuan_setData:uuid serviceDomain:zhiyuan_XYUUIDForKeyChain];
    return uuid;
}

+ (NSString *)zhiyuan_uuidForDevice {
    NSString *deviceUUID = [ZhiYuan_XYKeyChain zhiyuan_getDataWithServiceDomain:zhiyuan_XYUUIDForKeyChainAndDeviceOrIdfa];
    if (deviceUUID && ![deviceUUID isEqualToString:@""] && ![deviceUUID isEqualToString:zhiyuan_ZeroIdfa]) {
        return deviceUUID;
    }
    
    NSString *idfa = [self zhiyuan_uuidForIDFA];
    if (idfa && ![idfa isEqualToString:@""] && ![idfa isEqualToString:zhiyuan_ZeroIdfa]) {
        [ZhiYuan_XYKeyChain zhiyuan_setData:idfa serviceDomain:zhiyuan_XYUUIDForKeyChainAndDeviceOrIdfa];
        return idfa;
    }
    
    NSString *deviceInfoUUID = [self zhiyuan_uuidForDeviceInfo];
    if (deviceInfoUUID && ![deviceInfoUUID isEqualToString:@""]) {
        [ZhiYuan_XYKeyChain zhiyuan_setData:deviceInfoUUID serviceDomain:zhiyuan_XYUUIDForKeyChainAndDeviceOrIdfa];
        return deviceInfoUUID;
    }
    
    NSString *uuid = [self zhiyuan_uuid];
    if (uuid && ![uuid isEqualToString:@""]) {
        [ZhiYuan_XYKeyChain zhiyuan_setData:uuid serviceDomain:zhiyuan_XYUUIDForKeyChainAndDeviceOrIdfa];
        return uuid;
    }
    
    return @"";
}

@end
